function [waitrate,jumpcdfout,jumpbincent] = cdfs(upp, dbw, spf, bins, ymax, Ny, jumps)

    allwaits = []; % Store for the jump wait times  
    alljumps = []; % Store for the jump sizes
    waitrate = []; % Store for the jump wait time cdf  
    jumpcdf = []; % Store for the jump size cdf
    un = unique(jumps(:,7)); % List of particle indicies
    [N,~] = size(un); % Number of particles
    jumpbins = linspace(-ymax,ymax,Ny); % Length CDF bins
    jumpbincent = jumpbins(1:end-1) + (jumpbins(2) - jumpbins(1))/2; % Center of cdf bins
    
    %%% Jump rates
    % get all the waits
    for i = 1:N 
        a = find(jumps(:,7) == un(i)); % Get trajectory for particle with index un(i).
        [jnum,~] = size(a); % Get number of steps.
        waits = zeros(jnum-1,2); % Preallocate wait array.
        if ~isempty(a) && jnum > 1 % Ensure trajectory is length > 2.
            waits(:,1) = jumps(a(2:end),5) - jumps(a(1:end-1),6); % Get the wait time between consecutive jumps.
            waits(:,2) = jumps(a(1:end-1),4); % Assign the position of jump's end as the 'wait' position.
        end
        allwaits = [allwaits;waits]; % Collect up all the waits.
    end
    
    % Dimensionalize, shift the y position and symmetrize
    allwaits(:,1) = allwaits(:,1)*spf; % Convert to seconds
    allwaits(:,2) = allwaits(:,2)*upp; % Convert to um
    may = max(allwaits(:,2)); miy = min(allwaits(:,2));   
    allwaits(:,2) = allwaits(:,2) - miy - (may-miy)/2; % Shift channel midpoint to 0
    allwaits( allwaits(:,2) < 0, 2) = -allwaits( allwaits(:,2) < 0, 2); % Symmetrize wrt center
    
    %%% Jump distributions
    % get all the jump y projections
    alljumps(:,1) = jumps(:,4) - jumps(:,3); % Jump size y-projection.
    alljumps(:,2) = jumps(:,3); % Jump distance from boundary at start of jump.
    
    % Dimensionalize, shift the y position and symmetrize
    alljumps(:,1:2) = upp*alljumps(:,1:2); % dimensionalise.
    may = max(alljumps(:,2)); miy = min(alljumps(:,2)); % Get max and min jump locations to symmetrise data.
    alljumps(:,2) = alljumps(:,2) - miy - (may-miy)/2; % Shift channel midpoint to 0.
    alljumps( alljumps(:,2) < 0, 2) = -alljumps( alljumps(:,2) < 0, 2); % Symmetrize wrt center.
    
    % Get the CDFs for the jump sizes for different bins.
    for i = 1:bins
       
        ymin = (i-1)*dbw; % Lower bound of the bin.
        ymax = (i)*dbw; % Upper bound of the bin.
        
        % Get the relevant data
        a1 = find(alljumps(:,2) >= ymin);
        b1 = find(alljumps(a1,2) < ymax);
        b1 = a1(b1);
        a2 = find(allwaits(:,2) >= ymin);
        b2 = find(allwaits(a2,2) < ymax);
        b2 = a2(b2);

        % Get the CDFs.
        jdcdf = histcounts(alljumps(b1,1),jumpbins,'normalization','cdf'); % Get the PDF of jump sizes.
        waitrate(i) = 1/mean(allwaits(b2,1)); % Get the local rate.
        jumpcdf = [jumpcdf; jdcdf]; % Concatenate to the master array to be output.
        
    end
    
    % Format the data for outputting.
    jumpcdf0 = jumpcdf;
    jumpcdf1 = flipud(fliplr(1 - jumpcdf));    
    jumpcdfout = [jumpcdf1; jumpcdf0];
    
end

