
%%% CODE TO SAMPLE THE EXPERIMENTAL DISTRIBUTIONS AND SIMMULATE JUMP DIFFUSION 
% Author: Stephen Williams
% History: 
% Notes: 
% Possible error: Requires you to make a directory 'outputs/'.

clear all
clear workspace

%%%%%%%%%%%
% User inputs.
diffon = 1; % Diffusion on (1) or off (0)?
rateposon = 1; % Spatially dependent rate, yes (1) or no (0)?
jumpposon = 1; % Spatially dependent jumps, yes (1) or no (0)?

%%%%%%%%%%%
% Constants.
N = 200000; % Simmulated steps.
dt = 0.1; % Time step.
runs = 100; % Number of colloids to simmulate.
% Loop on the different channel widths you're interested in.
for wid = 50:50:50 % Channel widths (um).
    rcol = 5; % Colloid radius (um).
    cmin = -wid/2+1; % Minimum accessible position (um).
    cmax = wid/2-1; % Maximum accessible position (um).
    if diffon == 1
        D0 = 0.05; % Thermal diffusivity (um^2/s).
    else
        D0 = 0; % Thermal diffusivity (um^2/s).
    end
    tDdt = sqrt(2*D0*dt); % Diffusion size.
    dbw = 1; % Distribution bin width (um).
    bins = wid/(2*dbw); % Number of bins.
    bincent = (0:bins-1)*dbw + dbw/2; % Get the bin center positions for the channel.
    bincentq = [-((bins-1:-1:0)*dbw + dbw/2), (0:bins-1)*dbw + dbw/2]; % Get the bin center positions for the channel with negatives included
    upp = 0.55; % um per pixel (um).
    spf = 0.1; % seconds per frame (s).
    ymax = 50; Ny = 1000; % Length CDF constants.
    
    %%%%%%%%%%%
    % Load the detected jumps from experiments.
    load(['Jump_data/JumpStats_' num2str(wid) '.mat'])
    jumps = jumpstats1;
    
    %%%%%%%%%%%
    % Initialise arrays.
    pcol = zeros(N,runs); % Colloid positons.
    pcol(1,:) = (rand(1,runs)-0.5)*(wid-rcol*2); % Uniform colloid initial condition.
    bcol = zeros(1,runs); % Colloid bin Allocation poisson.
    bcolq = zeros(1,runs); % Colloid bin Allocation exp.
    pois = zeros(N,3); % Array to determine if jumps occur.
    wb = []; % Store for the wait bins.
    jb = []; % Store for the jump bins.
    lambdaTemp = zeros(1,runs); % Store for the determined local jump rate.
    jumpInds = zeros(1,runs); % Store for index of jcb chosen for each colloid.
    jumpSize = zeros(1,runs); % Value at that store.
    overshootval = zeros(1,runs); % Diffusive overshoot value.
    overshootb = zeros(1,runs); % Binarised value to determine if reflection is needed.
    [lambda,q,jbc] = cdfs(upp, dbw, spf, bins, ymax, Ny, jumps); % Get cdf's for lambda and Q.
    
    if rateposon == 0 % Turn off positionally dependent rate if specified.
        lambda(:) = lambda(ones(1,runs));
    end
    
    %%%%%%%%%%%
    % Loop on time steps.
    for t = 2:N
        [~,bcol] = min(abs(abs(pcol(t-1,:)) - bincent(:))); % Get the inital bin number of every colloid.
        if jumpposon == 1
            [~,bcolq] = min(abs(pcol(t-1,:) - bincentq(:))); % Get the inital bin number of every colloid.
        else
            [~,bcolq] = ones(1,length(bcolq)); % Have all colloids act as though in bulk.
        end
        pois = (sign(1-exp(-lambda(bcol).*dt)-rand(1,runs)) + 1)/2; % Work out which colloids jump.
        urandn = rand(1,runs); % Get a list of uniformly distributed numbers.
        [~,jumpInds] = min(abs(q(bcolq,:) - urandn(:)),[],2); % Use bin # and list to generate list of jumps
        jumpSize = sign(pcol(t-1,:)).*jbc(jumpInds); % Get the jump sizes from the jump cdf bin centers, flip sign of jumps in negative side.
        pcol(t,:) = pcol(t-1,:) + (abs(pois-1)).*tDdt.*randn(1,runs) + pois.*jumpSize; % Let the system evolve 1 timestep.
        overshootval = (abs(pois-1)).*( pcol(t,:) - sign(pcol(t,:)).*cmax ); % Get the signed distance to the nearest boundary.
        overshootb = (abs(pois-1)).*abs( sign(pcol(t,:)) + sign(overshootval) )/2; % Use the sign comparison of the prev. value and the current position to estalish if a reflection is needed.
        pcol(t,:) = pois.*min(max(pcol(t,:),cmin),cmax) + (abs(pois-1)).*(pcol(t,:)-2.*overshootval.*overshootb); % Implement both boundary conditions.
    end
    
    %%%%%%%%%%%
    % Process and save the pdf of the simulation
    pcolp = abs(pcol);
    [pdf,edges] = histcounts(pcolp,wid,'normalization','pdf');
    cent = edges(1:end-1) + (edges(2) - edges(1))/2;
    pdf = [pdf;cent];
    pdf(1,:) = pdf(1,:)/sum(pdf(1,:));
    % save(['outputs/pdf_' num2str(wid)], 'pdf'); % Uncomment to save outputs.
end

%%%%%%%%%%%
% Optional Plotting functions
%% Full Channel Plot
histogram(pcol)
%% Half Channel Plot
histogram(pcolp,100,'normalization','pdf')
%%%%%%%%%%%