function [pos,Dkm,Veff,store,M2,Lmbd] = Dr2()

    % calculate thermal diffusivity.
    kb = 1.3806488*10^(-23); % Boltzmann constant.
    Tempe = 300; % Kelvin temp.
    nu = 8.9 * 10^(-4); % dynamic visc of water.
    rm = 3*10^(-6); % colloid radius (m).
    Dt = (10^12) * kb*Tempe/(6*pi*nu*rm); % Thermal diffusivity (um^2/s).

    % Fitting parameters for L+/- and \lambda (fig. 3 inset, red/blue solid lines, fig 2b, black solid line)
    % See Methods section A.  
    %Note: Y_col is ommited due to use of effective system radius.
    rcol = 3; % Colloid radius.
    rsys = 200-rcol; % System effective radius.
    L = 0.375; Lb = 3.70; lpo = 5.05; lne = 10.8;
    tw = 0.237; tb = 0.315; lt = 8.20;  
    
    dx = 1; % Drift/diffusion array resolution
    
    % Initialise stores for outputs
    pos = (0:dx:rsys)'; N = length(pos); % Positions
    Lp1 = zeros(N,1); Lm1 = zeros(N,1); % L±
    T1 = zeros(N,1); Lmbd = zeros(N,1); % rates
    M1 = zeros(N,1); M2 = zeros(N,1); % Jump distribution moments
    Dkm = zeros(N,1); Veff = zeros(N,1); % Drift/diffusivity

    % Calculate the values for the different positions.
    % See Methods section A for equations.
    for i = 1:N
        Lp1(i)  = ((L - Lb)*exp(-(rsys - pos(i))/lpo)) + Lb;
        Lm1(i) = ((L - Lb)*exp(-(rsys - pos(i))/lne)) + Lb;
        T1(i) = ((tw - tb)*exp(-(rsys - pos(i))/lt)) + tb;
        M1(i) = -(Lp1(i) - Lm1(i));
        M2(i) = 2*(Lp1(i)^3 + Lm1(i)^3)/(Lp1(i) + Lm1(i)); 
        Lmbd(i) = T1(i);
        Lmbd(i) = Lmbd(i) * (3.8/4.5); % adjust for relative concentration.
        Dkm(i) =  Dt + (Lmbd(i)*M2(i)/2);     
    end
    
    % Use values to get Veff values.
    for i = 1:N
        Veff(i) = Lmbd(i)*M1(i);
    end
    
    % Non-Dimensionalise the values.
    Veff = Veff/rsys;
    Dkm = Dkm/(rsys^2);
    
end

