% Brownian Escape Model - Milstein Algorithm %

%%%%%%%%%%%%%%%%%%%%%
% Constants and initalisation.
clear all
%
n = 10000; % Number of colloids.
pos = zeros(n,4); % Default initial condition at chamber center.
% Uniform initial condition.
% pos(:,3) = rand(n,1); % unif. random radii
% pos(:,4) = 2*pi*(rand(n,1)); % unif. random angle
% pos(:,1) = pos(:,3).*cos(pos(:,4)); % reconstruct positions.
% pos(:,2) = pos(:,3).*sin(pos(:,4)); % reconstruct positions.

% Initial position 2, TBD.
% get K-M prob dist. and then inverse sample CDF to get positions.

texit = zeros(n,1); % Array for particle exit times.
checker = zeros(n,1); % Boundary condition value store.
checker2 = zeros(n,1); % Absorb condition store.
checker3 = zeros(n,1);  % Reflect condition store.
R = 1; % Normalised chamber radius.
rcol = 3; % Colloid radius (um).
Rsys = 200-rcol; % Chamber radius (um).
dt = 0.1; % Time step (s).
T = 150000; % Total time (s).
dw = (7.5-2*rcol)/Rsys; % Exit size (um).
b = linspace(0,2*pi,15); % Set of exit centres (14 total).
b = [b(2:7),b(9:14)]; % Remove 2 exits, to match experimental geometry.
[lpos,lD,lV,~] = Dr2(); % Get the local drift/diffusion values.
a = zeros(n,1); % Store for colloid's positions indexes.
lDp = gradient(lD,1/Rsys); % Get diffuvity gradient for Milstein correction drift.
oa = ones(n,2); % Ito

%%%%%%%%%%%%%%%%%%%%%

for t = dt:dt:T
    % Get particle positions for drift/diffusion values.
    for j = 1:length(pos)
        [~,a(j)] = min(abs(Rsys*pos(j,3) - lpos(:)));
    end
    % Calculate particle moves.
    u = randn(n,2); % Get an array of normally distributed random numbers.
    dx = sqrt(2.*lD(a).*dt).*u; % Construct diffusion.
    vdt = lV(a).*dt.*[cos(pos(:,4)),sin(pos(:,4))]; % Cosntruct the particle's drift.
    mcr = -0.5.*lDp(a).*dt.* (u.^2 - oa).*[cos(pos(:,4)),sin(pos(:,4))]; % Milstein method correction drift.
    % Implement moves.
    pos(:,1:2) = pos(:,1:2) + (dx+vdt+mcr).*(1-sign(checker)); % Evolve the particle.
    pos(:,3) = sqrt(pos(:,1).^2 + pos(:,2).^2); % Get the particle's distance from the center for boundary conditions.        
    pos(:,4) = angle(pos(:,1)+1i*pos(:,2)) - 2*pi*(sign(angle(pos(:,1)+1i*pos(:,2)))-1)/2; % Get particle's angle relative to center.
    % Implement boundary conditions.
    checker2 = 1-max(-round(((sign(abs(pos(:,4) - b) - dw/2))-1)/2),[],2); % Check if the particle needs to be absored: 1=no,0=yes.
    checker3 = (sign(pos(:,3) - R)+1)./2; % Check if particle is outside system: 1=yes, 0=no.
    checker = checker + checker3 - checker2.*checker3; % Get the boundary condition for each particle to be implemeted.
    dr = (pos(:,3)-R).*checker2.*checker3; % Reflection contribution if outside system and not absorbed.
    pos(:,3) = pos(:,3) - 2*dr; % Reflect particle if required using radius from center.
    pos(:,1) = pos(:,3).*cos(pos(:,4)); % reconstruct reflected positions.
    pos(:,2) = pos(:,3).*sin(pos(:,4)); % reconstruct reflected positions.
    
end

%%%%%%%%%%%%%%%%%%%%%
%
% Get MFPT.
texit = T - checker*dt;
texit = texit(texit ~= T);
mfpt = 1/(mean(texit));
%
% Get statistics.
rate = 1./texit;
std = sqrt(var(rate));
sterr = std./sqrt(n);
%
%%%%%%%%%%%%%%%%%%%%%

% Plotting tools.
disp(['The escape rate is: ' num2str(mfpt)])
h = histogram(texit,'Normalization','pdf');
title('12 exit system')
xlabel('t (s)');
ylabel('P(t)');
dim = [.6 .7 .20 .1];
str = num2str(mfpt);
annotation('textbox',dim,'String',['Rate = ' str ' s^{-1}'])
%save('mist_texit_ito','texit') % uncomment to save output figure.